% ----------------------------------------------------------------------- %
%                                                                         %
%  FFFFFFFFFFF DDDDDDDDD  CCCCCCCCCC FFFFFFFFFFF IIIIIIIIIIII TTTTTTTTTT  %
%  FFFFFFFFFFF DDDDDDDDDD CCCCCCCCC  FFFFFFFFFFF  IIIIIIIIII  TTTTTTTTTT  %
%  FF          DD      DD CC         FF               II          TT      %
%  FF          DD      DD CC         FF               II          TT      %
%  FFFFFF      DD      DD CC         FFFFFF           II          TT      % 
%  FF          DD      DD CC         FF               II          TT      %
%  FF          DDDDDDDDDD CCCCCCCCC  FF           IIIIIIIIII      TT      %
%  FF          DDDDDDDDD  CCCCCCCCCC FF          IIIIIIIIIIII     TT      %
%                                                                         %
% ----------------------------------------------------------------------- %

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% This code determines the fitting coefficients of the water retention    %
% functions of van Genuchten (VG) and Kosugi to model the flow duration   %
% curves. Four different optimization methods are implemented to locate   %
% quickly the optimum parameter values. The user should use each of these %
% methods to make sure that optimum values of the fitting coefificents    %
% are found ( my experience has shown that this is sometimes difficult )  %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% For more details regarding this work refer to the two following papers  %
%                                                                         %
% 1. Vrugt, J.A., and M. Sadegh (2013), Toward diagnostic model           %
%    calibration and evaluation: Approximate Bayesian computation, Water  %
%    Resources Research, 49, 43354345, doi:10.1002/wrcr.20354.           %
% 2. Sadegh, M., J.A. Vrugt, X. Cu, and H.V. Gupta, (2016), The soil      %
%    water characteristic as new class of parametric expressions of the   %
%    flow duration curve, Journal of Hydrology, p. 1-19, In Press,        %
%    doi:10.1016/j.jhydrol.2016.01.027.                                   %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% SYNOPSIS      [ x , RMSE ] = FDCFIT ( FDCPar , Meas_info );             %
% SYNOPSIS      [ x , RMSE ] = FDCFIT ( FDCPar , Meas_info , optim );     %
%                                                                         %
% INPUT:        FDCPar      structure with settings for FDCFIT            %
%               Meas_info   structure with empirical data of              %
%               optim       optional structure with optimization settings %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     Written by Jasper A. Vrugt                          %
%                   University of California Irvine                       %
%                           December 2014                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% Version 1: December, 2014                                               %
% Version 1.1: Feb, 2016                                                  %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Close everything
clear all; clc; close all hidden;

% Which of the case studies should the code run
case_study = 2;                     % Which case study to solve?

% Store working directory and subdirectory containing the files needed to run each example
EXAMPLE_dir = [pwd '\case_study_' num2str(case_study)]; addpath(EXAMPLE_dir);

% The following models are part of the toolbox
% "model"   "name"      "acronym used in paper Sadegh, Vrugt (REF 2)"
% [ 1 ] 'lognormal-2'    LN-2
% [ 2 ] 'gumbel'         G
% [ 3 ] 'logistic'       LG
% [ 4 ] 'logarithmic'    LOG
% [ 5 ] 'power'          PW
% [ 6 ] 'quimpo'         Q
% [ 7 ] 'viola'          V
% [ 8 ] 'genuchten-2'    VG-2
% [ 9 ] 'kosugi-2'       K-2
% [ 10 ] 'lognormal-3'   LN-3
% [ 11 ] 'pareto'        GP
% [ 12 ] 'gev'           GEV
% [ 13 ] 'franchini'     F
% [ 14 ] 'genuchten-3'   VG-3
% [ 15 ] 'kosugi-3'      K-3

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %                         SETUP OF PROBLEM                            % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

% Which class of model and formulation? (defined by user)
FDCPar.model = 15;                  % Which model do you want to use?
                                    % Input: [1...15]   ( See list above )
FDCPar.formulation = 'Y';           % Fitting approach (fit to observed exceedance probability or streamflow)
                                    % Input: 'E'        Against exceedance probability
                                    %        'Y'        Against streamflow
FDCPar.method = 'LM';               % Optimization method used to derive the coefficients of the FDC model
                                    % Options:  'LM'    Levenberg Marquardt                                ( = local gradient-based method )
                                    %           'SP'    Nelder-Mead Simplex                                ( = local method )
                                    %           'CMA'   Covariance Matrix Adaptation Evolutionary Strategy ( = global method )
                                    %           'DE'    Differential Evolution                             ( = global method )
% Define fields of structure optim                                                                    
optim.N = 5;                        % Number of trials with Levenberg Marquardt/Simplex method             ( default: 5)
optim.P = 25;                       % DE or CMAES Population size                                          ( default: 25 )
optim.CR = 0.8;                     % DE Crossover value                                                   ( default: 0.8 )
optim.TolX = 1e-2;                  % Termination criteria on parameters                                   ( default: 0.01 )
optim.TolFun = 1e-3;                % Termination criteria on objective function ( SSE )                   ( default: 0.001 )
optim.MaxFunEvals = 2e4;            % Maximum number of function evaluations with each method (per trial)  ( default: 2e4 )

% Define fields of structure options
options.type = 'daily';             % Time scale of flow duration curve ( 'daily'/'weekly'/'monthly'/'annual' )
options.print = 'yes';              % Output to screen ( 'yes' or 'no' )

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %                        CASE STUDIES: DEFINE DATA                    % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

% Now check which case study to do
switch case_study
    
    case {1}, % Case Study 1: French Broad River - wettest of MOPEX data set
        
        % Load data of watershed
        ID_watershed = '08167500';
        
        % Now compute the daily empirical FDC (discharge in column 6)
        [ E , Y , p_0 ] = CalcFDC ( ID_watershed , options.type , 6 );
        
    case {2}, % Case Study 2: Guadalupe River basin - driest of MOPEX data set
        
        % Load data of watershed
        ID_watershed = '03443000';
        
        % Now compute the daily empirical FDC (discharge in column 6)
        [ E , Y , p_0 ] = CalcFDC ( ID_watershed , options.type , 6 );

    case {3}, % Case Study 3: MOPEX data set
        
        % Do FDC fitting: watershed_no = [1...50] (too much space for all MOPEX data sets)
        watershed_no = 10; 
        
        % Load the data of basin j
        evalstr = strcat('data = load(''data_',num2str(watershed_no),'.txt'');'); eval(evalstr);

        % Now compute the daily empirical FDC (discharge in column 1)
        [ E , Y , p_0 ] = CalcFDC ( data , options.type , 1 );
        
end

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %                      DERIVE FITTING COEFFICIENTS                    % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
[ x , RMSE ] = FDCFIT ( FDCPar , E , Y , optim , options );